@@ -1,6 +1,6 @@
 -- |
 -- Module      :  Control.Applicative.Combinators
--- Copyright   :  © 2017 Mark Karpov
+-- Copyright   :  © 2017–present Mark Karpov
 -- License     :  BSD 3 clause
 --
 -- Maintainer  :  Mark Karpov <markkarpov92@gmail.com>
@@ -12,6 +12,16 @@
 -- commonly used in parsing from "Control.Applicative" with additional
 -- parsing-related comments added.
 --
+-- Due to the nature of the 'Applicative' and 'Alternative' abstractions,
+-- they are prone to memory leaks and not as efficient as their monadic
+-- counterparts. Although all the combinators we provide in this module are
+-- perfectly expressible in terms of 'Applicative' and 'Alternative', please
+-- prefer "Control.Monad.Combinators" instead when possible.
+--
+-- If you wish that the combinators that cannot return empty lists return
+-- values of the 'Data.List.NonEmpty.NonEmpty' data type, use the
+-- "Control.Applicative.Combinators.NonEmpty" module.
+--
 -- === A note on backtracking
 --
 -- Certain parsing libraries, such as Megaparsec, do not backtrack every
@@ -24,16 +34,16 @@
 -- cannot know anything about inner workings of any concrete parsing
 -- library, and so they cannot use @try@.
 --
--- An essential feature of the 'Alternative' type class is the @('<|>')@
+-- The essential feature of the 'Alternative' type class is the @('<|>')@
 -- operator that allows to express choice. In libraries that do not
 -- backtrack everything automatically, the choice operator and everything
--- that is build on top of it require the parser of the left hand side to
+-- that is build on top of it require the parser on the left hand side to
 -- backtrack in order for the alternative branch of parsing to be tried.
 -- Thus it is the responsibility of the programmer to wrap more complex,
 -- composite parsers in @try@ to achieve correct behavior.
 
 {-# LANGUAGE BangPatterns #-}
-{-# LANGUAGE CPP          #-}
+{-# LANGUAGE TupleSections #-}
 
 module Control.Applicative.Combinators
   ( -- * Re-exports from "Control.Applicative"
@@ -45,6 +55,8 @@ module Control.Applicative.Combinators
     -- $some
   , optional
     -- $optional
+  , empty
+    -- $empty
 
     -- * Original combinators
   , between
@@ -55,7 +67,9 @@ module Control.Applicative.Combinators
   , endBy
   , endBy1
   , manyTill
+  , manyTill_
   , someTill
+  , someTill_
   , option
   , sepBy
   , sepBy1
@@ -69,13 +83,8 @@ module Control.Applicative.Combinators
 where
 
 import Control.Applicative
-import Data.Foldable
-
-#if MIN_VERSION_base(4,9,0)
 import Control.Monad (replicateM, replicateM_)
-#elif !MIN_VERSION_base(4,8,0)
-import Data.Traversable (sequenceA)
-#endif
+import Data.Foldable
 
 ----------------------------------------------------------------------------
 -- Re-exports from "Control.Applicative"
@@ -89,14 +98,14 @@ import Data.Traversable (sequenceA)
 -- $many
 --
 -- @'many' p@ applies the parser @p@ /zero/ or more times and returns a list
--- of the returned values of @p@.
+-- of the values returned by @p@.
 --
 -- > identifier = (:) <$> letter <*> many (alphaNumChar <|> char '_')
 
 -- $some
 --
 -- @'some' p@ applies the parser @p@ /one/ or more times and returns a list
--- of the returned values of @p@.
+-- of the values returned by @p@.
 --
 -- > word = some letter
 
@@ -109,6 +118,13 @@ import Data.Traversable (sequenceA)
 --
 -- See also: 'option'.
 
+-- $empty
+--
+-- This parser fails unconditionally without providing any information about
+-- the cause of the failure.
+--
+-- @since 0.4.0
+
 ----------------------------------------------------------------------------
 -- Original combinators
 
@@ -123,22 +139,23 @@ between open close p = open *> p <* close
 
 -- | @'choice' ps@ tries to apply the parsers in the list @ps@ in order,
 -- until one of them succeeds. Returns the value of the succeeding parser.
+--
+-- > choice = asum
 
 choice :: (Foldable f, Alternative m) => f (m a) -> m a
 choice = asum
 {-# INLINE choice #-}
 
 -- | @'count' n p@ parses @n@ occurrences of @p@. If @n@ is smaller or equal
--- to zero, the parser equals to @'pure' []@. Returns a list of @n@ values.
+-- to zero, the parser equals to @'pure' []@. Returns a list of @n@ parsed
+-- values.
+--
+-- > count = replicateM
 --
 -- See also: 'skipCount', 'count''.
 
 count :: Applicative m => Int -> m a -> m [a]
-#if MIN_VERSION_base(4,9,0)
 count = replicateM
-#else
-count n p = sequenceA (replicate n p)
-#endif
 {-# INLINE count #-}
 
 -- | @'count'' m n p@ parses from @m@ to @n@ occurrences of @p@. If @n@ is
@@ -148,7 +165,7 @@ count n p = sequenceA (replicate n p)
 -- Please note that @m@ /may/ be negative, in this case effect is the same
 -- as if it were equal to zero.
 --
--- See also: 'count'.
+-- See also: 'skipCount', 'count'.
 
 count' :: Alternative m => Int -> Int -> m a -> m [a]
 count' m' n' p = go m' n'
@@ -160,6 +177,8 @@ count' m' n' p = go m' n'
 {-# INLINE count' #-}
 
 -- | Combine two alternatives.
+--
+-- > eitherP a b = (Left <$> a) <|> (Right <$> b)
 
 eitherP :: Alternative m => m a -> m b -> m (Either a b)
 eitherP a b = (Left <$> a) <|> (Right <$> b)
@@ -182,7 +201,8 @@ endBy1 p sep = some (p <* sep)
 {-# INLINE endBy1 #-}
 
 -- | @'manyTill' p end@ applies parser @p@ /zero/ or more times until parser
--- @end@ succeeds. Returns the list of values returned by @p@.
+-- @end@ succeeds. Returns the list of values returned by @p@. @end@ result
+-- is consumed and lost. Use 'manyTill_' if you wish to keep it.
 --
 -- See also: 'skipMany', 'skipManyTill'.
 
@@ -192,8 +212,24 @@ manyTill p end = go
     go = ([] <$ end) <|> liftA2 (:) p go
 {-# INLINE manyTill #-}
 
+-- | @'manyTill_' p end@ applies parser @p@ /zero/ or more times until
+-- parser @end@ succeeds. Returns the list of values returned by @p@ and the
+-- @end@ result. Use 'manyTill' if you have no need in the result of the
+-- @end@.
+--
+-- See also: 'skipMany', 'skipManyTill'.
+--
+-- @since 1.2.0
+
+manyTill_ :: Alternative m => m a -> m end -> m ([a], end)
+manyTill_ p end = go
+  where
+    go = (([],) <$> end) <|> liftA2 (\x (xs, y) -> (x:xs, y)) p go
+{-# INLINE manyTill_ #-}
+
 -- | @'someTill' p end@ works similarly to @'manyTill' p end@, but @p@
--- should succeed at least once.
+-- should succeed at least once. @end@ result is consumed and lost. Use
+-- 'someTill_' if you wish to keep it.
 --
 -- See also: 'skipSome', 'skipSomeTill'.
 
@@ -201,11 +237,24 @@ someTill :: Alternative m => m a -> m end -> m [a]
 someTill p end = liftA2 (:) p (manyTill p end)
 {-# INLINE someTill #-}
 
+-- | @'someTill_' p end@ works similarly to @'manyTill_' p end@, but @p@
+-- should succeed at least once. Use 'someTill' if you have no need in the
+-- result of the @end@.
+--
+-- See also: 'skipSome', 'skipSomeTill'.
+--
+-- @since 1.2.0
+
+someTill_ :: Alternative m => m a -> m end -> m ([a], end)
+someTill_ p end =
+  liftA2 (\x (xs, y) -> (x:xs, y)) p (manyTill_ p end)
+{-# INLINE someTill_ #-}
+
 -- | @'option' x p@ tries to apply the parser @p@. If @p@ fails without
 -- consuming input, it returns the value @x@, otherwise the value returned
 -- by @p@.
 --
--- > priority = option 0 (digitToInt <$> digitChar)
+-- > option x p = p <|> pure x
 --
 -- See also: 'optional'.
 
@@ -263,20 +312,18 @@ skipSome :: Alternative m => m a -> m ()
 skipSome p = p *> skipMany p
 {-# INLINE skipSome #-}
 
--- | @'skipCount' n p@ parses @n@ occurrences of @p@, skipping its result. If
--- @n@ is smaller or equal to zero, the parser equals to @'pure' []@. Returns a
--- list of @n@ values.
+-- | @'skipCount' n p@ parses @n@ occurrences of @p@, skipping its result.
+-- If @n@ is not positive, the parser equals to @'pure' []@. Returns a list
+-- of @n@ values.
+--
+-- > skipCount = replicateM_
 --
 -- See also: 'count', 'count''.
 --
 -- @since 0.3.0
 
 skipCount :: Applicative m => Int -> m a -> m ()
-#if MIN_VERSION_base(4,9,0)
 skipCount = replicateM_
-#else
-skipCount n p = sequenceA_ (replicate n p)
-#endif
 {-# INLINE skipCount #-}
 
 -- | @'skipManyTill' p end@ applies the parser @p@ /zero/ or more times
